'use strict';
var _ = require('underscore');
var joi = require('joi');
var Foxx = require('org/arangodb/foxx');
var ArangoError = require('org/arangodb').ArangoError;
var IssueRepo = require('repositories/issue');
var Issue = require('models/issue');
var controller = new Foxx.Controller(applicationContext);
var console = require('console');

var issueIdSchema = joi.string().required()
.description('The id of the issue')
.meta({allowMultiple: false});

var issueRepo = new IssueRepo(
  applicationContext.collection('issue'),
  {model: Issue}
);

/** Lists of all issue.
 *
 * This function simply returns the list of all Issue.
 */
controller.get('/', function (req, res) {
	var filter = req.params('filter');
	var sortFld = req.params('sort');
	var full = req.params('fullsearch');
	var sort = null, sortJoin = null;
	//console.log('Filter: ', filter);
	if(sortFld) {
		var sf = sortFld.split(" ");
		switch(sf[0]) {
		case 'project':
		case 'category':
			sort = 'y.name';
			sortJoin = 'for y in ' + applicationContext.mount.substring(1) + '_' + sf[0] + ' filter y._key==x.' + sf[0] + ' ';
			break;
		case 'status':
		case 'priority':
			sort = "'y.sort'";
			sortJoin = 'for y in ' + applicationContext.mount.substring(1) + '_' + sf[0] + ' filter y._key==x.' + sf[0] + ' ';
			break;
		default:
			sort = 'x.' + sf[0];
			break;
		}
		if(sort !== null && sf.length > 1)
			sort = sort + ' ' + sf[sf.length-1];
	}	
	var ftxt = null;
	if(full && full !== null && full.trim().length > 0) {
		ftxt = ' filter (x._key in (for t in fulltext(simpleITS_issue, "description", "' + full + '") return t._key) || ' +
			'x._key in (for z in fulltext(simpleITS_reply, "text", "' + full + '") return z.issueKey)) '; 
	}
	if(filter || sortFld || ftxt !== null) {
		var lst = null;
		var query = 'for x in ' + applicationContext.mount.substring(1) + '_issue ' +
			(ftxt !== null ? ftxt : '') + 
			(sortJoin !== null ? sortJoin : '') +
			(filter && sortJoin == null && ftxt == null ? ' filter ' : (filter ? ' && ' : ' ')) + 
			(filter ? filter : '') +
			(sort !== null ? ' sort ' + sort : '') +
			' return x';
		console.log(query);
		lst = Foxx.createQuery(query)();
		if(lst !== null) {
			res.json(lst);
			return;
		}
	}
	res.json(_.map(issueRepo.all(), function (model) {
		return model.forClient();
	}));
})
.queryParam('filter', {
  description: 'Filter for issue list',
  type: joi.string().optional()
})
.queryParam('sort', {
  description: 'Sorting for issue list',
  type: joi.string().optional()
})
.queryParam('fullsearch', {
  description: 'Fullsearch for issue list',
  type: joi.string().optional()
});

/** Creates a new issue.
 *
 * Creates a new issue. The information has to be in the
 * requestBody.
 */
controller.post('/', function (req, res) {
  var issue = req.parameters.issue;
  var lst = Foxx.createQuery('upsert { type: @type} insert { type: @type, curNo: 1} ' + 
		'update { curNo: has(OLD, "endNo") && OLD.curNo + 1 > OLD.endNo ? has(OLD, "startNo") ? OLD.startNo : 1 '+
		': has(OLD, "startNo") && OLD.curNo + 1 < OLD.startNo ? OLD.startNo : OLD.curNo + 1} in simpleITS_sitsno return NEW');
	var erg = lst({type: 'IssueNo'});
	issue.attributes.issueNo = erg[0].curNo;
  res.json(issueRepo.save(issue).forClient());
})
.bodyParam('issue', {
  description: 'The issue you want to create',
  type: Issue
});

/** Reads a issue.
 *
 * Reads a issue.
 */
controller.get('/:id', function (req, res) {
  var id = req.urlParameters.id;
  res.json(issueRepo.byId(id).forClient());
})
.pathParam('id', issueIdSchema)
.errorResponse(ArangoError, 404, 'The issue could not be found');

/** Replaces a issue.
 *
 * Changes a issue. The information has to be in the
 * requestBody.
 */
controller.put('/:id', function (req, res) {
  var id = req.urlParameters.id;
  var issue = req.parameters.issue;
  res.json(issueRepo.replaceById(id, issue));
})
.pathParam('id', issueIdSchema)
.bodyParam('issue', {
  description: 'The issue you want your old one to be replaced with',
  type: Issue
})
.errorResponse(ArangoError, 404, 'The issue could not be found');

/** Updates a issue.
 *
 * Changes a issue. The information has to be in the
 * requestBody.
 */
controller.patch('/:id', function (req, res) {
  var id = req.urlParameters.id;
  var patchData = req.parameters.patch;
  res.json(issueRepo.updateById(id, patchData));
})
.pathParam('id', issueIdSchema)
.bodyParam('patch', {
  description: 'The patch data you want your issue to be updated with',
  type: joi.object().required()
})
.errorResponse(ArangoError, 404, 'The issue could not be found');

/** Removes a issue.
 *
 * Removes a issue.
 */
controller.delete('/:id', function (req, res) {
  var id = req.urlParameters.id;
  issueRepo.removeById(id);
  res.json({success: true});
})
.pathParam('id', issueIdSchema)
.errorResponse(ArangoError, 404, 'The issue could not be found');
