'use strict';
var _ = require('underscore');
var joi = require('joi');
var Foxx = require('org/arangodb/foxx');
var ArangoError = require('org/arangodb').ArangoError;
var PriorityRepo = require('repositories/priority');
var Priority = require('models/priority');
var controller = new Foxx.Controller(applicationContext);

var priorityIdSchema = joi.string().required()
.description('The id of the priority')
.meta({allowMultiple: false});

var priorityRepo = new PriorityRepo(
  applicationContext.collection('priority'),
  {model: Priority}
);

/** Lists of all priority.
 *
 * This function simply returns the list of all Priority.
 */
controller.get('/', function (req, res) {
  res.json(_.map(priorityRepo.all(), function (model) {
    return model.forClient();
  }));
});

/** Creates a new priority.
 *
 * Creates a new priority. The information has to be in the
 * requestBody.
 */
controller.post('/', function (req, res) {
  var priority = req.parameters.priority;
  res.json(priorityRepo.save(priority).forClient());
})
.bodyParam('priority', {
  description: 'The priority you want to create',
  type: Priority
});

/** Reads a priority.
 *
 * Reads a priority.
 */
controller.get('/:id', function (req, res) {
  var id = req.urlParameters.id;
  res.json(priorityRepo.byId(id).forClient());
})
.pathParam('id', priorityIdSchema)
.errorResponse(ArangoError, 404, 'The priority could not be found');

/** Replaces a priority.
 *
 * Changes a priority. The information has to be in the
 * requestBody.
 */
controller.put('/:id', function (req, res) {
  var id = req.urlParameters.id;
  var priority = req.parameters.priority;
  res.json(priorityRepo.replaceById(id, priority));
})
.pathParam('id', priorityIdSchema)
.bodyParam('priority', {
  description: 'The priority you want your old one to be replaced with',
  type: Priority
})
.errorResponse(ArangoError, 404, 'The priority could not be found');

/** Updates a priority.
 *
 * Changes a priority. The information has to be in the
 * requestBody.
 */
controller.patch('/:id', function (req, res) {
  var id = req.urlParameters.id;
  var patchData = req.parameters.patch;
  res.json(priorityRepo.updateById(id, patchData));
})
.pathParam('id', priorityIdSchema)
.bodyParam('patch', {
  description: 'The patch data you want your priority to be updated with',
  type: joi.object().required()
})
.errorResponse(ArangoError, 404, 'The priority could not be found');

/** Removes a priority.
 *
 * Removes a priority.
 */
controller.delete('/:id', function (req, res) {
  var id = req.urlParameters.id;
  priorityRepo.removeById(id);
  res.json({success: true});
})
.pathParam('id', priorityIdSchema)
.errorResponse(ArangoError, 404, 'The priority could not be found');

/** Reads priority of category.
 *
 * Reads priority of category.
 */
controller.get('/stat/:id', function (req, res) {
  var id = req.urlParameters.id;
  var q = Foxx.createQuery('for x in ' + applicationContext.mount.substring(1) + 
  	'_issue filter x.priority==@id collect priority=x.priority with count into anz return {status: priority, count: anz}');
  var lst = q({id: id});
  res.json(lst);
})
.pathParam('id', priorityIdSchema)
.errorResponse(ArangoError, 404, 'The priority statistic could not be found');
