'use strict';
var _ = require('underscore');
var joi = require('joi');
var Foxx = require('org/arangodb/foxx');
var ArangoError = require('org/arangodb').ArangoError;
var ReplyRepo = require('repositories/reply');
var Reply = require('models/reply');
var controller = new Foxx.Controller(applicationContext);
var console = require('console');

var replyIdSchema = joi.string().required()
.description('The id of the reply')
.meta({allowMultiple: false});

var replyRepo = new ReplyRepo(
  applicationContext.collection('reply'),
  {model: Reply}
);

/** Lists of all reply.
 *
 * This function simply returns the list of all Reply.
 */
controller.get('/', function (req, res) {
	var filter = req.params('filter');
	console.log('Filter: ', filter);
	if(filter) {
		var lst = null;
		lst = Foxx.createQuery('for x in ' + applicationContext.mount.substring(1) + '_reply filter ' + filter + ' sort x.createDate return x')();
		if(lst !== null) {
			res.json(lst);
			return;
		}
	}
  res.json(_.map(replyRepo.all(), function (model) {
    return model.forClient();
  }));
})
.queryParam('filter', {
  description: 'Filter for issue list',
  type: joi.string().optional()
});

/** Creates a new reply.
 *
 * Creates a new reply. The information has to be in the
 * requestBody.
 */
controller.post('/', function (req, res) {
  var reply = req.parameters.reply;
  res.json(replyRepo.save(reply).forClient());
})
.bodyParam('reply', {
  description: 'The reply you want to create',
  type: Reply
});

/** Reads a reply.
 *
 * Reads a reply.
 */
controller.get('/:id', function (req, res) {
  var id = req.urlParameters.id;
  res.json(replyRepo.byId(id).forClient());
})
.pathParam('id', replyIdSchema)
.errorResponse(ArangoError, 404, 'The reply could not be found');

/** Replaces a reply.
 *
 * Changes a reply. The information has to be in the
 * requestBody.
 */
controller.put('/:id', function (req, res) {
  var id = req.urlParameters.id;
  var reply = req.parameters.reply;
  res.json(replyRepo.replaceById(id, reply));
})
.pathParam('id', replyIdSchema)
.bodyParam('reply', {
  description: 'The reply you want your old one to be replaced with',
  type: Reply
})
.errorResponse(ArangoError, 404, 'The reply could not be found');

/** Updates a reply.
 *
 * Changes a reply. The information has to be in the
 * requestBody.
 */
controller.patch('/:id', function (req, res) {
  var id = req.urlParameters.id;
  var patchData = req.parameters.patch;
  res.json(replyRepo.updateById(id, patchData));
})
.pathParam('id', replyIdSchema)
.bodyParam('patch', {
  description: 'The patch data you want your reply to be updated with',
  type: joi.object().required()
})
.errorResponse(ArangoError, 404, 'The reply could not be found');

/** Removes a reply.
 *
 * Removes a reply.
 */
controller.delete('/:id', function (req, res) {
  var id = req.urlParameters.id;
  replyRepo.removeById(id);
  res.json({success: true});
})
.pathParam('id', replyIdSchema)
.errorResponse(ArangoError, 404, 'The reply could not be found');
