'use strict';
var _ = require('underscore');
var joi = require('joi');
var Foxx = require('org/arangodb/foxx');
var ArangoError = require('org/arangodb').ArangoError;
var SitsnoRepo = require('repositories/sitsno');
var Sitsno = require('models/sitsno');
var controller = new Foxx.Controller(applicationContext);

var sitsnoIdSchema = joi.string().required()
.description('The id of the sitsno')
.meta({allowMultiple: false});

var sitsnoRepo = new SitsnoRepo(
  applicationContext.collection('sitsno'),
  {model: Sitsno}
);

/** Lists of all sitsno.
 *
 * This function simply returns the list of all Sitsno.
 */
controller.get('/', function (req, res) {
  res.json(_.map(sitsnoRepo.all(), function (model) {
    return model.forClient();
  }));
});

/** Creates a new sitsno.
 *
 * Creates a new sitsno. The information has to be in the
 * requestBody.
 */
controller.post('/', function (req, res) {
  var sitsno = req.parameters.sitsno;
  res.json(sitsnoRepo.save(sitsno).forClient());
})
.bodyParam('sitsno', {
  description: 'The sitsno you want to create',
  type: Sitsno
});

/** Reads a sitsno.
 *
 * Reads a sitsno.
 */
controller.get('/:id', function (req, res) {
  var id = req.urlParameters.id;
  res.json(sitsnoRepo.byId(id).forClient());
})
.pathParam('id', sitsnoIdSchema)
.errorResponse(ArangoError, 404, 'The sitsno could not be found');

/** Replaces a sitsno.
 *
 * Changes a sitsno. The information has to be in the
 * requestBody.
 */
controller.put('/:id', function (req, res) {
  var id = req.urlParameters.id;
  var sitsno = req.parameters.sitsno;
  res.json(sitsnoRepo.replaceById(id, sitsno));
})
.pathParam('id', sitsnoIdSchema)
.bodyParam('sitsno', {
  description: 'The sitsno you want your old one to be replaced with',
  type: Sitsno
})
.errorResponse(ArangoError, 404, 'The sitsno could not be found');

/** Updates a sitsno.
 *
 * Changes a sitsno. The information has to be in the
 * requestBody.
 */
controller.patch('/:id', function (req, res) {
  var id = req.urlParameters.id;
  var patchData = req.parameters.patch;
  res.json(sitsnoRepo.updateById(id, patchData));
})
.pathParam('id', sitsnoIdSchema)
.bodyParam('patch', {
  description: 'The patch data you want your sitsno to be updated with',
  type: joi.object().required()
})
.errorResponse(ArangoError, 404, 'The sitsno could not be found');

/** Removes a sitsno.
 *
 * Removes a sitsno.
 */
controller.delete('/:id', function (req, res) {
  var id = req.urlParameters.id;
  sitsnoRepo.removeById(id);
  res.json({success: true});
})
.pathParam('id', sitsnoIdSchema)
.errorResponse(ArangoError, 404, 'The sitsno could not be found');
