'use strict';
var _ = require('underscore');
var joi = require('joi');
var Foxx = require('org/arangodb/foxx');
var ArangoError = require('org/arangodb').ArangoError;
var StatusRepo = require('repositories/status');
var Status = require('models/status');
var controller = new Foxx.Controller(applicationContext);

var statusIdSchema = joi.string().required()
.description('The id of the status')
.meta({allowMultiple: false});

var statusRepo = new StatusRepo(
  applicationContext.collection('status'),
  {model: Status}
);

/** Lists of all status.
 *
 * This function simply returns the list of all Status.
 */
controller.get('/', function (req, res) {
  res.json(_.map(statusRepo.all(), function (model) {
    return model.forClient();
  }));
});

/** Creates a new status.
 *
 * Creates a new status. The information has to be in the
 * requestBody.
 */
controller.post('/', function (req, res) {
  var status = req.parameters.status;
  res.json(statusRepo.save(status).forClient());
})
.bodyParam('status', {
  description: 'The status you want to create',
  type: Status
});

/** Reads a status.
 *
 * Reads a status.
 */
controller.get('/:id', function (req, res) {
  var id = req.urlParameters.id;
  res.json(statusRepo.byId(id).forClient());
})
.pathParam('id', statusIdSchema)
.errorResponse(ArangoError, 404, 'The status could not be found');

/** Replaces a status.
 *
 * Changes a status. The information has to be in the
 * requestBody.
 */
controller.put('/:id', function (req, res) {
  var id = req.urlParameters.id;
  var status = req.parameters.status;
  res.json(statusRepo.replaceById(id, status));
})
.pathParam('id', statusIdSchema)
.bodyParam('status', {
  description: 'The status you want your old one to be replaced with',
  type: Status
})
.errorResponse(ArangoError, 404, 'The status could not be found');

/** Updates a status.
 *
 * Changes a status. The information has to be in the
 * requestBody.
 */
controller.patch('/:id', function (req, res) {
  var id = req.urlParameters.id;
  var patchData = req.parameters.patch;
  res.json(statusRepo.updateById(id, patchData));
})
.pathParam('id', statusIdSchema)
.bodyParam('patch', {
  description: 'The patch data you want your status to be updated with',
  type: joi.object().required()
})
.errorResponse(ArangoError, 404, 'The status could not be found');

/** Removes a status.
 *
 * Removes a status.
 */
controller.delete('/:id', function (req, res) {
  var id = req.urlParameters.id;
  statusRepo.removeById(id);
  res.json({success: true});
})
.pathParam('id', statusIdSchema)
.errorResponse(ArangoError, 404, 'The status could not be found');

/** Reads statistic of status.
 *
 * Reads statistic of status.
 */
controller.get('/stat/:id', function (req, res) {
  var id = req.urlParameters.id;
  var q = Foxx.createQuery('for x in ' + applicationContext.mount.substring(1) + 
  	'_issue filter x.status==@id collect status=x.status with count into anz return {status: status, count: anz}');
  var lst = q({id: id});
  res.json(lst);
})
.pathParam('id', statusIdSchema)
.errorResponse(ArangoError, 404, 'The status statistic could not be found');
